/* ------------------------------------------------------------------------------*/
/* LIDL.SO                                                                       */
/* Vereinfachter Aufruf einiger LAPACK Routinen aus IDL                          */
/* (c) 8/2000 by Andreas Reigber                                                 */
/* ------------------------------------------------------------------------------*/
/* INHALT :                                                                      */
/* c_eigen_all : eigenvalues and eigenvectors of arbitrary n x n complex matrices*/
/* r_eigen_all : eigenvalues and eigenvectors of arbitrary n x n real matrices*/
/* c_eigen_hs  : c_eigen_hs : eigenvalues and eigenvectors of complex hermitian matrices*/
/* r_eigen_hs  : eigenvalues and eigenvectors of real symmetric matrices*/
/* c_svd       : singular value decomposition of arbitrary n x n complex matrices*/
/* r_svd       : singular value decomposition of arbitrary n x n real matrices*/
/* ------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include "f2c.h"

void c_eigen_all(argc, argv)
int argc;
void *argv[];
{
	complex *a, *w, *vl, *vr, *work;
	float *rwork;
	int n,lda,ldvl,ldvr,lwork,info;
	int i;
	
	a          = (complex *)argv[0];	
	n          = *(int *)argv[1];
	lda        = n;
        w          = (complex *)argv[2];	
	ldvl       = n;
	ldvr       = n;
	vl         = malloc(ldvl * n * sizeof(complex));
	vr         = (complex *)argv[3];
	lwork      = 2*n;
	work       = malloc( lwork * sizeof(complex));
	rwork      = malloc( 2 * n * sizeof(float));

	cgeev_("N","V",&n,a,&lda,w,vl,&ldvl,vr,&ldvr,work,&lwork,rwork,&info);

	free(vl);
	free(work);
	free(rwork);
}

void r_eigen_all(argc, argv)
int argc;
void *argv[];
{
	float *a, *wr, *wi, *vl, *vr, *work;
	int n,lda,ldvl,ldvr,lwork,info;
	int i;
	
	a          = (float *)argv[0];	
	n          =  *(int *)argv[1];
	lda        = n;
   wr         = (float *)argv[2];	
   wi         = (float *)argv[3];	
	ldvl       = n;
	ldvr       = n;
	vl         = malloc(ldvl * n * sizeof(float));
	vr         = (float *)argv[4];
	lwork      = 4*n;
	work       = malloc( lwork * sizeof(float));

	sgeev_("N","V",&n,a,&lda,wr,wi,vl,&ldvl,vr,&ldvr,work,&lwork,&info);
	
	free(vl);
	free(work);
}

void c_eigen_hs(argc, argv)
int argc;
void *argv[];
{
	complex *a, *work;
	float *w, *rwork;
	int n,lda,lwork,info;
	int i;
	
	a          = (complex *)argv[0];	
	n          = *(int *)argv[1];
	lda        = n;
   w          = (float *)argv[2];	
	lwork      = 2*n-1;
	work       = malloc( lwork * sizeof(complex));
	rwork      = malloc( ((3 * n) - 2) * sizeof(float));

	cheev_("V","U",&n,a,&lda,w,work,&lwork,rwork,&info);

	free(work);
	free(rwork);
}

void r_eigen_hs(argc, argv)
int argc;
void *argv[];
{
	float *a, *w, *work;
	int n,lda,lwork,info;
	int i;
	
	a          = (float *)argv[0];	
	n          = *(int *)argv[1];
	lda        = n;
   w          = (float *)argv[2];	
	lwork      = 3*n-1;
	work       = malloc( lwork * sizeof(float));

	ssyev_("V","U",&n,a,&lda,w,work,&lwork,&info);

	free(work);
}

/* ---------------------------------------------------------------*/
/* SVD Dekomposition fuer beliebige komplexe Matritzen            */
/* ---------------------------------------------------------------*/
/* Aufruf:                                                        */
/* c_svd(a,m,n,s,u,vt,d1,d2)                                      */
/*   a = n x m Matrix (komplex)                                   */
/*   m = erste Dimension (integer)                                */
/*   n = zweite Dimension (integer)                               */
/*   s = sigular Values, fltarr Laenge min(n,m)                   */
/*   u = m x m Matrix mit singular vectors (komplex)              */
/*  vt = n x n Matrix mit singular vectors (komplex)              */
/*  d1 = 2*min(m,n)+max(m,n)               (integer)              */
/*  d2 = max(3*min(m,n),5*min(m,n)-4)      (integer)              */


void c_svd(argc, argv)
int argc;
void *argv[];
{
	complex *a, *u, *vt, *work;
	float *s, *rwork;
	int m,n,lda,ldu,ldvt,lwork,rwsiz,info;
	
	a          = (complex *)argv[0];	
	m          = *(int *)argv[1];
	n          = *(int *)argv[2];
	lda        = m;
   s          = (float *)argv[3];	
   u          = (complex *)argv[4];	
	ldu        = m;
   vt         = (complex *)argv[5];	
	ldvt       = n;
	lwork      = *(int *)argv[6];
	work       = malloc( lwork * sizeof(complex));
	rwsiz      = *(int *)argv[7];	
	rwork      = malloc( rwsiz * sizeof(float));

	cgesvd_("A","A",&m,&n,a,&lda,s,u,&ldu,vt,&ldvt,work,&lwork,rwork,&info);

	free(work);
	free(rwork);
}

/* ---------------------------------------------------------------*/
/* SVD Dekomposition fuer beliebige reele Matritzen               */
/* ---------------------------------------------------------------*/
/* Aufruf:                                                        */
/* r_svd(a,m,n,s,u,vt,d1,d2)                                      */
/*   a = n x m Matrix (reell)                                     */
/*   m = erste Dimension (integer)                                */
/*   n = zweite Dimension (integer)                               */
/*   s = sigular Values, fltarr Laenge min(n,m)                   */
/*   u = m x m Matrix (float) mit singular vectors                */
/*  vt = n x n Matrix (float) mit singular vectors                */
/*  d1 = max(3*min(m,n)+max(m,n),5*min(m,n)-4)  (integer)         */
void r_svd(argc, argv)
int argc;
void *argv[];
{
	float *a, *u, *s, *vt, *work;
	int m,n,lda,ldu,ldvt,lwork,rwsiz,info;
	
	a          = (float *)argv[0];	
	m          = *(int *)argv[1];
	n          = *(int *)argv[2];
	lda        = m;
   s          = (float *)argv[3];	
   u          = (float *)argv[4];	
	ldu        = m;
   vt         = (float *)argv[5];	
	ldvt       = n;
	lwork      = *(int *)argv[6];
	work       = malloc( lwork * sizeof(float));

	sgesvd_("A","A",&m,&n,a,&lda,s,u,&ldu,vt,&ldvt,work,&lwork,&info);

	free(work);
}


